#pragma once

#include "constants.h"

#include <essentials/types.h>

#include <cmath>

using namespace Essentials;

namespace Math
{
	float Sqrt(float x);
	float Sin(float x);
	float Cos(float x);
	float Tan(float x);
	float ASin(float x);
	float ACos(float x);
	float ACos_Clamped(float x);
	float ATan2(float y, float x);

	template <class TYPE> TYPE Sqr(TYPE x);
	template <class TYPE> TYPE Max(TYPE a, TYPE b);
	template <class TYPE> TYPE Min(TYPE a, TYPE b);
	template <class TYPE> TYPE Clamp(TYPE x, TYPE a, TYPE b);
	template <class TYPE> bool IsPowerOfTwo(TYPE x);
	int Round(float x);
	float Frac(float x);
	float Saturate(float x);
	float Log2(float x);

	uint32 Idx(uint32 x, uint32 y, uint32 width);
	uint32 Idx(uint32 x, uint32 y, uint32 z, uint32 width, uint32 height);

	//

	inline float Sqrt(float x)
	{
		return sqrtf(x);
	}

	inline float Sin(float x)
	{
		return sinf(x);
	}

	inline float Cos(float x)
	{
		return cosf(x);
	}

	inline float Tan(float x)
	{
		return tanf(x);
	}

	inline float ASin(float x)
	{
		return asinf(x);
	}

	inline float ACos(float x)
	{
		return acosf(x);
	}

	inline float ACos_Clamped(float x)
	{
		if (x <= -1.0f)
			return Pi;
		else if (x >= 1.0f)
			return 0.0f;
		else
			return acosf(x);
	}

	inline float ATan2(float y, float x)
	{
		return atan2f(y, x);
	}

	template <class TYPE>
	inline TYPE Sqr(TYPE x)
	{
		return x * x;
	}

	template <class TYPE>
	inline TYPE Max(TYPE a, TYPE b)
	{
		return a > b ? a : b;
	}

	template <class TYPE>
	inline TYPE Min(TYPE a, TYPE b)
	{
		return a < b ? a : b;
	}

	template <class TYPE>
	inline TYPE Clamp(TYPE x, TYPE a, TYPE b)
	{
		return Max(Min(x, b), a);
	}

	template <class TYPE>
	inline bool IsPowerOfTwo(TYPE x)
	{
		return !(x & (x-1));
	}

	inline int Round(float x)
	{
		return (int)(x + 0.5f);
	}

	inline float Frac(float x)
	{
		return x - (int)x;
	}

	inline float Saturate(float x)
	{
		return Clamp(x, 0.0f, 1.0f);
	}

	inline float Log2(float x)
	{
		return logf(x) / logf(2.0f);
	}

	inline uint32 Idx(uint32 x, uint32 y, uint32 width)
	{
		return y*width + x;
	}

	inline uint32 Idx(uint32 x, uint32 y, uint32 z, uint32 width, uint32 height)
	{
		return y*width*height + x*height + z;
	}
}
