#pragma once

#include "constants.h"
#include "common.h"
#include "matrix.h"
#include "quaternion.h"

namespace Math
{
	float DegToRad(float degrees);
	float RadToDeg(float radians);

	Vector3 QuaternionToEulerAngles(const Quaternion& q);
	Matrix QuaternionToMatrix(const Quaternion& q);

	//

	inline float DegToRad(float degrees)
	{
		return (degrees * (Pi / 180.0f));
	}

	inline float RadToDeg(float radians)
	{
		return (radians * (180.0f / Pi));
	}

	inline Vector3 QuaternionToEulerAngles(const Quaternion& q)
	{
		Vector3 temp;

		temp.x = ATan2(2.0f*(q.x*q.w + q.y*q.z), 1.0f - 2.0f*(q.x*q.x + q.y*q.y));
		temp.y = ASin(2.0f*(q.y*q.w - q.x*q.z));
		temp.z = ATan2(2.0f*(q.z*q.w + q.x*q.y), 1.0f - 2.0f*(q.y*q.y + q.z*q.z));

		return temp;
	}

	inline Matrix QuaternionToMatrix(const Quaternion& q)
	{
		return MatrixRotate(Axis(q), Angle(q));
	}
}
