#pragma once

#include "types.h"

namespace Math
{
	Matrix Mul(const Matrix& m1, const Matrix& m2);
	void MulIn(Matrix& m1, const Matrix& m2);
	void Transpose(Matrix& m);
	void Invert(Matrix& m);
	void Orthogonalize3x3(Matrix& m);
	Matrix Transposed(const Matrix& m);
	Matrix Inverted(const Matrix& m);
	Matrix Orthogonalized3x3(const Matrix& m);
	Matrix operator * (const Matrix& m1, const Matrix& m2);

	void SetZeros(Matrix& m);
	void SetIdentity(Matrix& m);
	void SetTranslate(Matrix& m, float x, float y, float z);
	void SetTranslate(Matrix& m, const Vector3& v);
	void SetRotate(Matrix& m, float x, float y, float z, float angle);
	void SetRotate(Matrix& m, const Vector3& axis, float angle);
	void SetRotateX(Matrix& m, float angle);
	void SetRotateY(Matrix& m, float angle);
	void SetRotateZ(Matrix& m, float angle);
	void SetScale(Matrix& m, float x, float y, float z);
	void SetScale(Matrix& m, const Vector3& s);
	void SetReflect(Matrix& m, const Plane& plane);

	void SetLookAtLH(Matrix& m, const Vector3& eye, const Vector3& at, const Vector3& up);
	void SetLookAtRH(Matrix& m, const Vector3& eye, const Vector3& at, const Vector3& up);
	void SetPerspectiveFovLH(Matrix& m, ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar);
	void SetPerspectiveFovRH(Matrix& m, ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar);
	void SetOrthoOffCenterLH(Matrix& m, ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar);
	void SetOrthoOffCenterRH(Matrix& m, ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar);
	void SetOrthoLH(Matrix& m, ZRange::TYPE zRange, float width, float height, float zNear, float zFar);
	void SetOrthoRH(Matrix& m, ZRange::TYPE zRange, float width, float height, float zNear, float zFar);

	Matrix MatrixIdentity();
	Matrix MatrixTranslate(float x, float y, float z);
	Matrix MatrixTranslate(const Vector3& v);
	Matrix MatrixRotate(float x, float y, float z, float angle);
	Matrix MatrixRotate(const Vector3& axis, float angle);
	Matrix MatrixRotateX(float angle);
	Matrix MatrixRotateY(float angle);
	Matrix MatrixRotateZ(float angle);
	Matrix MatrixScale(float x, float y, float z);
	Matrix MatrixScale(const Vector3& s);
	Matrix MatrixReflect(const Plane& plane);

	Matrix MatrixLookAtLH(const Vector3& eye, const Vector3& at, const Vector3& up);
	Matrix MatrixLookAtRH(const Vector3& eye, const Vector3& at, const Vector3& up);
	Matrix MatrixPerspectiveFovLH(ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar);
	Matrix MatrixPerspectiveFovRH(ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar);
	Matrix MatrixOrthoOffCenterLH(ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar);
	Matrix MatrixOrthoOffCenterRH(ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar);
	Matrix MatrixOrthoLH(ZRange::TYPE zRange, float width, float height, float zNear, float zFar);
	Matrix MatrixOrthoRH(ZRange::TYPE zRange, float width, float height, float zNear, float zFar);

	float GetDeterminant(
		float _00, float _01, float _02,
		float _10, float _11, float _12,
		float _20, float _21, float _22);

	//

	inline float GetDeterminant(
		float _00, float _01, float _02,
		float _10, float _11, float _12,
		float _20, float _21, float _22)
	{
		return ( (_00*_11*_22 + _10*_21*_02 + _20*_01*_12) - (_02*_11*_20 + _12*_21*_00 + _22*_01*_10) );
	}
}
