#pragma once

#include "types.h"
#include "vector.h"
#include "quaternion.h"

namespace Math
{
	Matrix Transform(const Matrix& m1, const Matrix& m2);
	Vector3 Transform(const Vector3& v, const Matrix& m);
	Vector3 Transform(const Matrix& m, const Vector3& v);
	Vector4 Transform(const Vector4& v, const Matrix& m);
	Vector4 Transform(const Matrix& m, const Vector4& v);
	Matrix operator * (const Matrix& m1, const Matrix& m2);
	Vector3 operator * (const Vector3& v, const Matrix& m);
	Vector3 operator * (const Matrix& m, const Vector3& v);
	Vector4 operator * (const Vector4& v, const Matrix& m);
	Vector4 operator * (const Matrix& m, const Vector4& v);

	Vector3 Transform(const Vector3& v, const Quaternion& q);

	Plane Transform(const Plane& plane, Matrix matrix);

	//

	inline Matrix Transform(const Matrix& m1, const Matrix& m2)
	{
		Matrix temp;

		for (int i = 0; i < 4; i++)
		{
			for (int j = 0; j < 4; j++)
			{
				temp._[i][j] =
					m1._[i][0] * m2._[0][j] +
					m1._[i][1] * m2._[1][j] +
					m1._[i][2] * m2._[2][j] +
					m1._[i][3] * m2._[3][j];
			}
		}

		return temp;
	}

	inline Vector3 Transform(const Vector3& v, const Matrix& m)
	{
		Vector3 temp;

		temp.x = v.x*m._[0][0] + v.y*m._[1][0] + v.z*m._[2][0];
		temp.y = v.x*m._[0][1] + v.y*m._[1][1] + v.z*m._[2][1];
		temp.z = v.x*m._[0][2] + v.y*m._[1][2] + v.z*m._[2][2];

		return temp;
	}

	inline Vector3 Transform(const Matrix& m, const Vector3& v)
	{
		Vector3 temp;

		temp.x = m._[0][0]*v.x + m._[0][1]*v.y + m._[0][2]*v.z;
		temp.y = m._[1][0]*v.x + m._[1][1]*v.y + m._[1][2]*v.z;
		temp.z = m._[2][0]*v.x + m._[2][1]*v.y + m._[2][2]*v.z;

		return temp;
	}

	inline Vector4 Transform(const Vector4& v, const Matrix& m)
	{
		Vector4 temp;

		temp.x = v.x*m._[0][0] + v.y*m._[1][0] + v.z*m._[2][0] + v.w*m._[3][0];
		temp.y = v.x*m._[0][1] + v.y*m._[1][1] + v.z*m._[2][1] + v.w*m._[3][1];
		temp.z = v.x*m._[0][2] + v.y*m._[1][2] + v.z*m._[2][2] + v.w*m._[3][2];
		temp.w = v.x*m._[0][3] + v.y*m._[1][3] + v.z*m._[2][3] + v.w*m._[3][3];

		return temp;
	}

	inline Vector4 Transform(const Matrix& m, const Vector4& v)
	{
		Vector4 temp;

		temp.x = m._[0][0]*v.x + m._[0][1]*v.y + m._[0][2]*v.z + m._[0][3]*v.w;
		temp.y = m._[1][0]*v.x + m._[1][1]*v.y + m._[1][2]*v.z + m._[1][3]*v.w;
		temp.z = m._[2][0]*v.x + m._[2][1]*v.y + m._[2][2]*v.z + m._[2][3]*v.w;
		temp.w = m._[3][0]*v.x + m._[3][1]*v.y + m._[3][2]*v.z + m._[3][3]*v.w;

		return temp;
	}

	inline Matrix operator * (const Matrix& m1, const Matrix& m2)
	{
		return Transform(m1, m2);
	}

	inline Vector3 operator * (const Vector3& v, const Matrix& m)
	{
		return Transform(v, m);
	}

	inline Vector3 operator * (const Matrix& m, const Vector3& v)
	{
		return Transform(m, v);
	}

	inline Vector4 operator * (const Vector4& v, const Matrix& m)
	{
		return Transform(v, m);
	}

	inline Vector4 operator * (const Matrix& m, const Vector4& v)
	{
		return Transform(m, v);
	}

	inline Vector3 Transform(const Vector3& v, const Quaternion& q)
	{
		Quaternion temp = (Conjugate(q)) * (QuaternionCustom(v.x, v.y, v.z, 0.0f)) * q;
		return VectorCustom(temp.x, temp.y, temp.z);
	}

	inline Plane Transform(const Plane& plane, Matrix matrix)
	{
		Vector4 v = VectorCustom(plane.a, plane.b, plane.c, plane.d);
		Invert(matrix);
		Transpose(matrix);
		v = v * matrix;

		Plane temp;

		temp.a = v.x;
		temp.b = v.y;
		temp.c = v.z;
		temp.d = v.w;

		Normalize(temp);

		return temp;
	}
}
