#pragma once

#include "types.h"
#include "common.h"

namespace Math
{
	// Vector2

	Vector2 VectorCustom(float x, float y);
	Vector2 VectorCopy(const Vector2& v);
	Vector2 Add(const Vector2& v1, const Vector2& v2);
	Vector2 Sub(const Vector2& v1, const Vector2& v2);
	Vector2 Mul(const Vector2& v1, const Vector2& v2);
	Vector2 Mul(const Vector2& v, float f);
	void AddIn(Vector2& v1, const Vector2& v2);
	void SubIn(Vector2& v1, const Vector2& v2);
	void MulIn(Vector2& v1, float f);
	float Dot(const Vector2& v1, const Vector2& v2);
	float LengthSquared(const Vector2& v);
	float Length(const Vector2& v);
	void SetLength(Vector2& v, float newLength);
	void Normalize(Vector2& v);
	Vector2 Normalized(const Vector2& v);
	float Angle(const Vector2& v1, const Vector2& v2);
	Vector2 Reflect(const Vector2& input, const Vector2& normal);
	Vector2 Slerp(const Vector2& v1, const Vector2& v2, float t);
	Vector2 operator + (const Vector2& v1, const Vector2& v2);
	Vector2 operator - (const Vector2& v1, const Vector2& v2);
	Vector2 operator * (const Vector2& v1, float f);
	Vector2 operator * (float f, const Vector2& v1);

	// Vector3

	Vector3 VectorCustom(float x, float y, float z);
	Vector3 VectorCopy(const Vector3& v);
	Vector3 Add(const Vector3& v1, const Vector3& v2);
	Vector3 Sub(const Vector3& v1, const Vector3& v2);
	Vector3 Mul(const Vector3& v1, const Vector3& v2);
	Vector3 Mul(const Vector3& v, float f);
	void AddIn(Vector3& v1, const Vector3& v2);
	void SubIn(Vector3& v1, const Vector3& v2);
	void MulIn(Vector3& v1, float f);
	float Dot(const Vector3& v1, const Vector3& v2);
	Vector3 Cross(const Vector3& v1, const Vector3& v2);
	float LengthSquared(const Vector3& v);
	float Length(const Vector3& v);
	void SetLength(Vector3& v, float newLength);
	void Normalize(Vector3& v);
	Vector3 Normalized(const Vector3& v);
	float Angle(const Vector3& v1, const Vector3& v2);
	Vector3 Reflect(const Vector3& input, const Vector3& normal);
	Vector3 Slerp(const Vector3& v1, const Vector3& v2, float t);
	Vector3 operator + (const Vector3& v1, const Vector3& v2);
	Vector3 operator - (const Vector3& v1, const Vector3& v2);
	Vector3 operator * (const Vector3& v1, float f);
	Vector3 operator * (float f, const Vector3& v1);

	// Vector4

	Vector4 VectorCustom(float x, float y, float z, float w);
	Vector4 VectorCopy(const Vector4& v);
	Vector4 Add(const Vector4& v1, const Vector4& v2);
	Vector4 Sub(const Vector4& v1, const Vector4& v2);
	Vector4 Mul(const Vector4& v1, const Vector4& v2);
	Vector4 Mul(const Vector4& v, float f);
	void AddIn(Vector4& v1, const Vector4& v2);
	void SubIn(Vector4& v1, const Vector4& v2);
	void MulIn(Vector4& v1, float f);
	void DivideByW(Vector4& v);
	Vector4 DividedByW(const Vector4& v);
	Vector4 operator + (const Vector4& v1, const Vector4& v2);
	Vector4 operator - (const Vector4& v1, const Vector4& v2);
	Vector4 operator * (const Vector4& v1, float f);
	Vector4 operator * (float f, const Vector4& v1);

	// Vector2

	inline Vector2 VectorCustom(float x, float y)
	{
		Vector2 temp;

		temp.x = x;
		temp.y = y;

		return temp;
	}

	inline Vector2 VectorCopy(const Vector2& v)
	{
		return VectorCustom(v.x, v.y);
	}

	inline Vector2 Add(const Vector2& v1, const Vector2& v2)
	{
		return VectorCustom(v1.x + v2.x, v1.y + v2.y);
	}

	inline Vector2 Sub(const Vector2& v1, const Vector2& v2)
	{
		return VectorCustom(v1.x - v2.x, v1.y - v2.y);
	}

	inline Vector2 Mul(const Vector2& v1, const Vector2& v2)
	{
		return VectorCustom(v1.x*v2.x, v1.y*v2.y);
	}

	inline Vector2 Mul(const Vector2& v, float f)
	{
		return VectorCustom(v.x*f, v.y*f);
	}

	inline void AddIn(Vector2& v1, const Vector2& v2)
	{
		v1.x += v2.x;
		v1.y += v2.y;
	}

	inline void SubIn(Vector2& v1, const Vector2& v2)
	{
		v1.x -= v2.x;
		v1.y -= v2.y;
	}

	inline void MulIn(Vector2& v1, float f)
	{
		v1.x *= f;
		v1.y *= f;
	}

	inline float Dot(const Vector2& v1, const Vector2& v2)
	{
		return v1.x*v2.x + v1.y*v2.y;
	}

	inline float LengthSquared(const Vector2& v)
	{
		return Dot(v, v);
	}

	inline float Length(const Vector2& v)
	{
		return fabs(LengthSquared(v));
	}

	inline void SetLength(Vector2& v, float newLength)
	{
		float scale = newLength / Length(v);

		v.x *= scale;
		v.y *= scale;
	}

	inline void Normalize(Vector2& v)
	{
		float oneOverLength = 1.0f / Length(v);

		v.x *= oneOverLength;
		v.y *= oneOverLength;
	}

	inline Vector2 Normalized(const Vector2& v)
	{
		Vector2 temp = VectorCopy(v);
		Normalize(temp);
		return temp;
	}

	inline float Angle(const Vector2& v1, const Vector2& v2)
	{
		return Cos(Dot(v1, v2));
	}

	inline Vector2 Reflect(const Vector2& input, const Vector2& normal)
	{
		return input - 2.0f*Dot(input, normal)*normal;
	}

	inline Vector2 Slerp(const Vector2& v1, const Vector2& v2, float t)
	{
		float omega = Angle(v1, v2);

		float s1 = Sin((1.0f - t)*omega);
		float s2 = Sin(t*omega);
		float s3 = Sin(omega);

		return v1*(s1/s3) + v2*(s2/s3);
	}

	inline Vector2 operator + (const Vector2& v1, const Vector2& v2)
	{
		return Add(v1, v2);
	}

	inline Vector2 operator - (const Vector2& v1, const Vector2& v2)
	{
		return Sub(v1, v2);
	}

	inline Vector2 operator * (const Vector2& v1, float f)
	{
		return Mul(v1, f);
	}

	inline Vector2 operator * (float f, const Vector2& v1)
	{
		return Mul(v1, f);
	}

	// Vector3

	inline Vector3 VectorCustom(float x, float y, float z)
	{
		Vector3 temp;

		temp.x = x;
		temp.y = y;
		temp.z = z;

		return temp;
	}

	inline Vector3 VectorCopy(const Vector3& v)
	{
		return VectorCustom(v.x, v.y, v.z);
	}

	inline Vector3 Add(const Vector3& v1, const Vector3& v2)
	{
		return VectorCustom(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
	}

	inline Vector3 Sub(const Vector3& v1, const Vector3& v2)
	{
		return VectorCustom(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
	}

	inline Vector3 Mul(const Vector3& v1, const Vector3& v2)
	{
		return VectorCustom(v1.x*v2.x, v1.y*v2.y, v1.z*v2.z);
	}

	inline Vector3 Mul(const Vector3& v, float f)
	{
		return VectorCustom(v.x*f, v.y*f, v.z*f);
	}

	inline void AddIn(Vector3& v1, const Vector3& v2)
	{
		v1.x += v2.x;
		v1.y += v2.y;
		v1.z += v2.z;
	}

	inline void SubIn(Vector3& v1, const Vector3& v2)
	{
		v1.x -= v2.x;
		v1.y -= v2.y;
		v1.z -= v2.z;
	}

	inline void MulIn(Vector3& v1, float f)
	{
		v1.x *= f;
		v1.y *= f;
		v1.z *= f;
	}

	inline float Dot(const Vector3& v1, const Vector3& v2)
	{
		return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
	}

	inline Vector3 Cross(const Vector3& v1, const Vector3& v2)
	{
		Vector3 temp;

		temp.x = v1.y*v2.z - v1.z*v2.y;
		temp.y = v1.z*v2.x - v1.x*v2.z;
		temp.z = v1.x*v2.y - v1.y*v2.x;

		return temp;
	}

	inline float LengthSquared(const Vector3& v)
	{
		return Dot(v, v);
	}

	inline float Length(const Vector3& v)
	{
		return Sqrt(LengthSquared(v));
	}

	inline void SetLength(Vector3& v, float newLength)
	{
		float scale = newLength / Length(v);

		v.x *= scale;
		v.y *= scale;
		v.z *= scale;
	}

	inline void Normalize(Vector3& v)
	{
		float oneOverLength = 1.0f / Length(v);

		v.x *= oneOverLength;
		v.y *= oneOverLength;
		v.z *= oneOverLength;
	}

	inline Vector3 Normalized(const Vector3& v)
	{
		Vector3 temp = VectorCopy(v);
		Normalize(temp);
		return temp;
	}

	inline float Angle(const Vector3& v1, const Vector3& v2)
	{
		return Cos(Dot(v1, v2));
	}

	inline Vector3 Reflect(const Vector3& input, const Vector3& normal)
	{
		return input - 2.0f*Dot(input, normal)*normal;
	}

	inline Vector3 Slerp(const Vector3& v1, const Vector3& v2, float t)
	{
		float omega = Angle(v1, v2);

		float s1 = Sin((1.0f - t) * omega);
		float s2 = Sin(t * omega);
		float s3 = Sin(omega);

		return v1*(s1/s3) + v2*(s2/s3);
	}

	inline Vector3 operator + (const Vector3& v1, const Vector3& v2)
	{
		return Add(v1, v2);
	}

	inline Vector3 operator - (const Vector3& v1, const Vector3& v2)
	{
		return Sub(v1, v2);
	}

	inline Vector3 operator * (const Vector3& v1, float f)
	{
		return Mul(v1, f);
	}

	inline Vector3 operator * (float f, const Vector3& v1)
	{
		return Mul(v1, f);
	}

	// Vector4

	inline Vector4 VectorCustom(float x, float y, float z, float w)
	{
		Vector4 temp;

		temp.x = x;
		temp.y = y;
		temp.z = z;
		temp.w = w;

		return temp;
	}

	inline Vector4 VectorCopy(const Vector4& v)
	{
		return VectorCustom(v.x, v.y, v.z, v.w);
	}

	inline Vector4 Add(const Vector4& v1, const Vector4& v2)
	{
		return VectorCustom(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z, v1.w + v2.w);
	}

	inline Vector4 Sub(const Vector4& v1, const Vector4& v2)
	{
		return VectorCustom(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z, v1.w - v2.w);
	}

	inline Vector4 Mul(const Vector4& v1, const Vector4& v2)
	{
		return VectorCustom(v1.x*v2.x, v1.y*v2.y, v1.z*v2.z, v1.w*v2.w);
	}

	inline Vector4 Mul(const Vector4& v, float f)
	{
		return VectorCustom(v.x*f, v.y*f, v.z*f, v.w*f);
	}

	inline void AddIn(Vector4& v1, const Vector4& v2)
	{
		v1.x += v2.x;
		v1.y += v2.y;
		v1.z += v2.z;
		v1.w += v2.w;
	}

	inline void SubIn(Vector4& v1, const Vector4& v2)
	{
		v1.x -= v2.x;
		v1.y -= v2.y;
		v1.z -= v2.z;
		v1.w -= v2.w;
	}

	inline void MulIn(Vector4& v1, float f)
	{
		v1.x *= f;
		v1.y *= f;
		v1.z *= f;
		v1.w *= f;
	}

	inline void DivideByW(Vector4& v)
	{
		float oneOverW = 1.0f / v.w;

		v.x *= oneOverW;
		v.y *= oneOverW;
		v.z *= oneOverW;
		v.w = 1.0f;
	}

	inline Vector4 DividedByW(const Vector4& v)
	{
		float oneOverW = 1.0f / v.w;
		return VectorCustom(v.x*oneOverW, v.y*oneOverW, v.z*oneOverW, 1.0f);
	}

	inline Vector4 operator + (const Vector4& v1, const Vector4& v2)
	{
		return Add(v1, v2);
	}

	inline Vector4 operator - (const Vector4& v1, const Vector4& v2)
	{
		return Sub(v1, v2);
	}

	inline Vector4 operator * (const Vector4& v1, float f)
	{
		return Mul(v1, f);
	}

	inline Vector4 operator * (float f, const Vector4& v1)
	{
		return Mul(v1, f);
	}
}
