#include <math/matrix.h>
#include <math/types.h>
#include <math/constants.h>
#include <math/vector.h>

Math::Matrix Math::Mul(const Matrix& m1, const Matrix& m2)
{
	Matrix temp;

	for (int i = 0; i < 4; i++)
	{
		for (int j = 0; j < 4; j++)
		{
			temp._[i][j] =
				m1._[i][0] * m2._[0][j] +
				m1._[i][1] * m2._[1][j] +
				m1._[i][2] * m2._[2][j] +
				m1._[i][3] * m2._[3][j];
		}
	}

	return temp;
}

void Math::MulIn(Matrix& m1, const Matrix& m2)
{
	m1 = Mul(m1, m2);
}

void Math::Transpose(Matrix& m)
{
	Matrix temp = m;

	for (int i = 0; i < 4; i++)
	{
		for (int j = 0; j < 4; j++)
		{
			m._[i][j] = temp._[j][i];
		}
	}
}

void Math::Invert(Matrix& m)
{
	float determinant =
		+m._[0][0] * GetDeterminant(
			m._[1][1], m._[1][2], m._[1][3],
			m._[2][1], m._[2][2], m._[2][3],
			m._[3][1], m._[3][2], m._[3][3])
		-m._[0][1] * GetDeterminant(
			m._[1][0], m._[1][2], m._[1][3],
			m._[2][0], m._[2][2], m._[2][3],
			m._[3][0], m._[3][2], m._[3][3])
		+m._[0][2] * GetDeterminant(
			m._[1][0], m._[1][1], m._[1][3],
			m._[2][0], m._[2][1], m._[2][3],
			m._[3][0], m._[3][1], m._[3][3])
		-m._[0][3] * GetDeterminant(
			m._[1][0], m._[1][1], m._[1][2],
			m._[2][0], m._[2][1], m._[2][2],
			m._[3][0], m._[3][1], m._[3][2]);

	if ( !(fabs(determinant) < Epsilon4) )
	{
		float adj[4][4];

		adj[0][0] = +GetDeterminant(
			m._[1][1], m._[1][2], m._[1][3],
			m._[2][1], m._[2][2], m._[2][3],
			m._[3][1], m._[3][2], m._[3][3]);
		adj[0][1] = -GetDeterminant(
			m._[1][0], m._[1][2], m._[1][3],
			m._[2][0], m._[2][2], m._[2][3],
			m._[3][0], m._[3][2], m._[3][3]);
		adj[0][2] = +GetDeterminant(
			m._[1][0], m._[1][1], m._[1][3],
			m._[2][0], m._[2][1], m._[2][3],
			m._[3][0], m._[3][1], m._[3][3]);
		adj[0][3] = -GetDeterminant(
			m._[1][0], m._[1][1], m._[1][2],
			m._[2][0], m._[2][1], m._[2][2],
			m._[3][0], m._[3][1], m._[3][2]);

		adj[1][0] = -GetDeterminant(
			m._[0][1], m._[0][2], m._[0][3],
			m._[2][1], m._[2][2], m._[2][3],
			m._[3][1], m._[3][2], m._[3][3]);
		adj[1][1] = +GetDeterminant(
			m._[0][0], m._[0][2], m._[0][3],
			m._[2][0], m._[2][2], m._[2][3],
			m._[3][0], m._[3][2], m._[3][3]);
		adj[1][2] = -GetDeterminant(
			m._[0][0], m._[0][1], m._[0][3],
			m._[2][0], m._[2][1], m._[2][3],
			m._[3][0], m._[3][1], m._[3][3]);
		adj[1][3] = +GetDeterminant(
			m._[0][0], m._[0][1], m._[0][2],
			m._[2][0], m._[2][1], m._[2][2],
			m._[3][0], m._[3][1], m._[3][2]);

		adj[2][0] = +GetDeterminant(
			m._[0][1], m._[0][2], m._[0][3],
			m._[1][1], m._[1][2], m._[1][3],
			m._[3][1], m._[3][2], m._[3][3]);
		adj[2][1] = -GetDeterminant(
			m._[0][0], m._[0][2], m._[0][3],
			m._[1][0], m._[1][2], m._[1][3],
			m._[3][0], m._[3][2], m._[3][3]);
		adj[2][2] = +GetDeterminant(
			m._[0][0], m._[0][1], m._[0][3],
			m._[1][0], m._[1][1], m._[1][3],
			m._[3][0], m._[3][1], m._[3][3]);
		adj[2][3] = -GetDeterminant(
			m._[0][0], m._[0][1], m._[0][2],
			m._[1][0], m._[1][1], m._[1][2],
			m._[3][0], m._[3][1], m._[3][2]);

		adj[3][0] = -GetDeterminant(
			m._[0][1], m._[0][2], m._[0][3],
			m._[1][1], m._[1][2], m._[1][3],
			m._[2][1], m._[2][2], m._[2][3]);
		adj[3][1] = +GetDeterminant(
			m._[0][0], m._[0][2], m._[0][3],
			m._[1][0], m._[1][2], m._[1][3],
			m._[2][0], m._[2][2], m._[2][3]);
		adj[3][2] = -GetDeterminant(
			m._[0][0], m._[0][1], m._[0][3],
			m._[1][0], m._[1][1], m._[1][3],
			m._[2][0], m._[2][1], m._[2][3]);
		adj[3][3] = +GetDeterminant(
			m._[0][0], m._[0][1], m._[0][2],
			m._[1][0], m._[1][1], m._[1][2],
			m._[2][0], m._[2][1], m._[2][2]);

		for (int i = 0; i < 4; i++)
		{
			for (int j = 0; j < 4; j++)
			{
				m._[i][j] = (1.0f / determinant) * adj[j][i];
			}
		}
	}
}

void Math::Orthogonalize3x3(Matrix& m)
{
	Vector3 v1 = VectorCustom(m._[0][0], m._[1][0], m._[2][0]);
	Vector3 v2 = VectorCustom(m._[0][1], m._[1][1], m._[2][1]);
	Vector3 v3 = VectorCustom(m._[0][2], m._[1][2], m._[2][2]);

	Vector3 u1 = v1;
	Vector3 u2 = v2 - (Dot(v2, u1)/Dot(u1, u1))*u1;
	Vector3 u3 = v3 - (Dot(v3, u1)/Dot(u1, u1))*u1 - (Dot(v3, u2)/Dot(u2, u2))*u2;

	m._[0][0] = u1.x;
	m._[0][1] = u2.x;
	m._[0][2] = u3.x;

	m._[1][0] = u1.y;
	m._[1][1] = u2.y;
	m._[1][2] = u3.y;

	m._[2][0] = u1.z;
	m._[2][1] = u2.z;
	m._[2][2] = u3.z;
}

Math::Matrix Math::Transposed(const Matrix& m)
{
	Matrix temp = m;
	Transpose(temp);
	return temp;
}

Math::Matrix Math::Inverted(const Matrix& m)
{
	Matrix temp = m;
	Invert(temp);
	return temp;
}

Math::Matrix Math::Orthogonalized3x3(const Matrix& m)
{
	Matrix temp = m;
	Orthogonalize3x3(temp);
	return temp;
}

Math::Matrix Math::operator * (const Matrix& m1, const Matrix& m2)
{
	return Mul(m1, m2);
}

void Math::SetZeros(Matrix& m)
{
	m._[0][0] = 0.0f;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = 0.0f;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	m._[2][2] = 0.0f;
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 0.0f;
}

void Math::SetIdentity(Matrix& m)
{
	m._[0][0] = 1.0f;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = 1.0f;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	m._[2][2] = 1.0f;
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetTranslate(Matrix& m, float x, float y, float z)
{
	m._[0][0] = 1.0f;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = x;

	m._[1][0] = 0.0f;
	m._[1][1] = 1.0f;
	m._[1][2] = 0.0f;
	m._[1][3] = y;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	m._[2][2] = 1.0f;
	m._[2][3] = z;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetTranslate(Matrix& m, const Vector3& v)
{
	SetTranslate(m, v.x, v.y, v.z);
}

void Math::SetRotate(Matrix& m, float x, float y, float z, float angle)
{
	float s = sinf(angle);
	float c = cosf(angle);

	m._[0][0] = c + x*x*(1-c);
	m._[0][1] = x*y*(1-c) - z*s;
	m._[0][2] = x*z*(1-c) + y*s;
	m._[0][3] = 0.0f;

	m._[1][0] = x*y*(1-c) + z*s;
	m._[1][1] = c + y*y*(1-c);
	m._[1][2] = y*z*(1-c) - x*s;
	m._[1][3] = 0.0f;

	m._[2][0] = x*z*(1-c) - y*s;
	m._[2][1] = y*z*(1-c) + x*s;
	m._[2][2] = c + z*z*(1-c);
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetRotate(Matrix& m, const Vector3& axis, float angle)
{
	SetRotate(m, axis.x, axis.y, axis.z, angle);
}

void Math::SetRotateX(Matrix& m, float angle)
{
	m._[0][0] = 1.0f;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = cosf(angle);
	m._[1][2] = -sin(angle);
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = sin(angle);
	m._[2][2] = cos(angle);
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetRotateY(Matrix& m, float angle)
{
	m._[0][0] = cos(angle);
	m._[0][1] = 0.0f;
	m._[0][2] = sin(angle);
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = 1.0f;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = -sin(angle);
	m._[2][1] = 0.0f;
	m._[2][2] = cos(angle);
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetRotateZ(Matrix& m, float angle)
{
	m._[0][0] = cos(angle);
	m._[0][1] = -sin(angle);
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = sin(angle);
	m._[1][1] = cos(angle);
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	m._[2][2] = 1.0f;
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetScale(Matrix& m, float x, float y, float z)
{
	m._[0][0] = x;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = y;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	m._[2][2] = z;
	m._[2][3] = 0.0f;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetScale(Matrix& m, const Vector3& s)
{
	SetScale(m, s.x, s.y, s.z);
}

void Math::SetReflect(Matrix& m, const Plane& plane)
{
	m._[0][0] = -2.0f*plane.a*plane.a + 1.0f;
	m._[0][1] = -2.0f*plane.a*plane.b;
	m._[0][2] = -2.0f*plane.a*plane.c;
	m._[0][3] = -2.0f*plane.a*plane.d;

	m._[1][0] = -2.0f*plane.b*plane.a;
	m._[1][1] = -2.0f*plane.b*plane.b + 1.0f;
	m._[1][2] = -2.0f*plane.b*plane.c;
	m._[1][3] = -2.0f*plane.b*plane.d;

	m._[2][0] = -2.0f*plane.c*plane.a;
	m._[2][1] = -2.0f*plane.c*plane.b;
	m._[2][2] = -2.0f*plane.c*plane.c + 1.0f;
	m._[2][3] = -2.0f*plane.c*plane.d;

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetLookAtLH(Matrix& m, const Vector3& eye, const Vector3& at, const Vector3& up)
{
	Vector3 zAxis = Normalized(at - eye);
	Vector3 xAxis = Normalized(Cross(up, zAxis));
	Vector3 yAxis = Cross(zAxis, xAxis);

	m._[0][0] = xAxis.x;
	m._[0][1] = xAxis.y;
	m._[0][2] = xAxis.z;
	m._[0][3] = -Dot(xAxis, eye);

	m._[1][0] = yAxis.x;
	m._[1][1] = yAxis.y;
	m._[1][2] = yAxis.z;
	m._[1][3] = -Dot(yAxis, eye);

	m._[2][0] = zAxis.x;
	m._[2][1] = zAxis.y;
	m._[2][2] = zAxis.z;
	m._[2][3] = -Dot(zAxis, eye);

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetLookAtRH(Matrix& m, const Vector3& eye, const Vector3& at, const Vector3& up)
{
	Vector3 zAxis = Normalized(eye - at);
	Vector3 xAxis = Normalized(Cross(up, zAxis));
	Vector3 yAxis = Cross(zAxis, xAxis);

	m._[0][0] = xAxis.x;
	m._[0][1] = xAxis.y;
	m._[0][2] = xAxis.z;
	m._[0][3] = -Dot(xAxis, eye);

	m._[1][0] = yAxis.x;
	m._[1][1] = yAxis.y;
	m._[1][2] = yAxis.z;
	m._[1][3] = -Dot(yAxis, eye);

	m._[2][0] = zAxis.x;
	m._[2][1] = zAxis.y;
	m._[2][2] = zAxis.z;
	m._[2][3] = -Dot(zAxis, eye);

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetPerspectiveFovLH(Matrix& m, ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar)
{
	float yScale = 1.0f / Tan(fovY / 2.0f);
	float xScale = yScale / aspectRatio;

	m._[0][0] = xScale;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = yScale;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = (zNear + zFar) / (zFar - zNear);
		m._[2][3] = -2.0f * zNear * zFar / (zFar - zNear);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = zFar / (zFar - zNear);
		m._[2][3] = -zNear * zFar / (zFar - zNear);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 1.0f;
	m._[3][3] = 0.0f;
}

void Math::SetPerspectiveFovRH(Matrix& m, ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar)
{
	float yScale = 1.0f / Tan(fovY / 2.0f);
	float xScale = yScale / aspectRatio;

	m._[0][0] = xScale;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = yScale;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = (zNear + zFar) / (zNear - zFar);
		m._[2][3] = 2.0f * zNear * zFar / (zNear - zFar);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = zFar / (zNear - zFar);
		m._[2][3] = zNear * zFar / (zNear - zFar);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = -1.0f;
	m._[3][3] = 0.0f;
}

void Math::SetOrthoOffCenterLH(Matrix& m, ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar)
{
	m._[0][0] = 2.0f / (right - left);
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = (1.0f + right) / (1.0f - right);

	m._[1][0] = 0.0f;
	m._[1][1] = 2.0f / (top - bottom);
	m._[1][2] = 0.0f;
	m._[1][3] = (top + bottom) / (bottom - top);

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = -2.0f / (zNear - zFar);
		m._[2][3] = (zNear + zFar) / (zNear - zFar);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = 1.0f / (zFar - zNear);
		m._[2][3] = -zNear / (zFar - zNear);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetOrthoOffCenterRH(Matrix& m, ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar)
{
	m._[0][0] = 2.0f / (right - left);
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = (1.0f + right) / (1.0f - right);

	m._[1][0] = 0.0f;
	m._[1][1] = 2.0f / (top - bottom);
	m._[1][2] = 0.0f;
	m._[1][3] = (top + bottom) / (bottom - top);

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = 2.0f / (zNear - zFar);
		m._[2][3] = (zNear + zFar) / (zNear - zFar);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = 1.0f / (zNear - zFar);
		m._[2][3] = zNear / (zNear - zFar);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetOrthoLH(Matrix& m, ZRange::TYPE zRange, float width, float height, float zNear, float zFar)
{
	m._[0][0] = 2.0f / width;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = 2.0f / height;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = -2.0f / (zNear - zFar);
		m._[2][3] = (zNear + zFar) / (zNear - zFar);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = 1.0f / (zFar - zNear);
		m._[2][3] = -zNear / (zFar - zNear);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

void Math::SetOrthoRH(Matrix& m, ZRange::TYPE zRange, float width, float height, float zNear, float zFar)
{
	m._[0][0] = 2.0f / width;
	m._[0][1] = 0.0f;
	m._[0][2] = 0.0f;
	m._[0][3] = 0.0f;

	m._[1][0] = 0.0f;
	m._[1][1] = 2.0f / height;
	m._[1][2] = 0.0f;
	m._[1][3] = 0.0f;

	m._[2][0] = 0.0f;
	m._[2][1] = 0.0f;
	if (zRange == ZRange::MinusOneToPlusOne)
	{
		m._[2][2] = 2.0f / (zNear - zFar);
		m._[2][3] = (zNear + zFar) / (zNear - zFar);
	}
	else if (zRange == ZRange::ZeroToOne)
	{
		m._[2][2] = 1.0f / (zNear - zFar);
		m._[2][3] = zNear / (zNear - zFar);
	}

	m._[3][0] = 0.0f;
	m._[3][1] = 0.0f;
	m._[3][2] = 0.0f;
	m._[3][3] = 1.0f;
}

Math::Matrix Math::MatrixIdentity()
{
	Matrix temp;
	SetIdentity(temp);
	return temp;
}

Math::Matrix Math::MatrixTranslate(float x, float y, float z)
{
	Matrix temp;
	SetTranslate(temp, x, y, z);
	return temp;
}

Math::Matrix Math::MatrixTranslate(const Vector3& v)
{
	return MatrixTranslate(v.x, v.y, v.z);
}

Math::Matrix Math::MatrixRotate(float x, float y, float z, float angle)
{
	Matrix temp;
	SetRotate(temp, x, y, z, angle);
	return temp;
}

Math::Matrix Math::MatrixRotate(const Vector3& axis, float angle)
{
	return MatrixRotate(axis.x, axis.y, axis.z, angle);
}

Math::Matrix Math::MatrixRotateX(float angle)
{
	Matrix temp;
	SetRotateX(temp, angle);
	return temp;
}

Math::Matrix Math::MatrixRotateY(float angle)
{
	Matrix temp;
	SetRotateY(temp, angle);
	return temp;
}

Math::Matrix Math::MatrixRotateZ(float angle)
{
	Matrix temp;
	SetRotateZ(temp, angle);
	return temp;
}

Math::Matrix Math::MatrixScale(float x, float y, float z)
{
	Matrix temp;
	SetScale(temp, x, y, z);
	return temp;
}

Math::Matrix Math::MatrixScale(const Vector3& s)
{
	return MatrixScale(s.x, s.y, s.z);
}

Math::Matrix Math::MatrixReflect(const Plane& plane)
{
	Matrix temp;
	SetReflect(temp, plane);
	return temp;
}

Math::Matrix Math::MatrixLookAtLH(const Vector3& eye, const Vector3& at, const Vector3& up)
{
	Matrix temp;
	SetLookAtLH(temp, eye, at, up);
	return temp;
}

Math::Matrix Math::MatrixLookAtRH(const Vector3& eye, const Vector3& at, const Vector3& up)
{
	Matrix temp;
	SetLookAtRH(temp, eye, at, up);
	return temp;
}

Math::Matrix Math::MatrixPerspectiveFovLH(ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar)
{
	Matrix temp;
	SetPerspectiveFovLH(temp, zRange, fovY, aspectRatio, zNear, zFar);
	return temp;
}

Math::Matrix Math::MatrixPerspectiveFovRH(ZRange::TYPE zRange, float fovY, float aspectRatio, float zNear, float zFar)
{
	Matrix temp;
	SetPerspectiveFovRH(temp, zRange, fovY, aspectRatio, zNear, zFar);
	return temp;
}

Math::Matrix Math::MatrixOrthoOffCenterLH(ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar)
{
	Matrix temp;
	SetOrthoOffCenterLH(temp, zRange, left, right, bottom, top, zNear, zFar);
	return temp;
}

Math::Matrix Math::MatrixOrthoOffCenterRH(ZRange::TYPE zRange, float left, float right, float bottom, float top, float zNear, float zFar)
{
	Matrix temp;
	SetOrthoOffCenterRH(temp, zRange, left, right, bottom, top, zNear, zFar);
	return temp;
}

Math::Matrix Math::MatrixOrthoLH(ZRange::TYPE zRange, float width, float height, float zNear, float zFar)
{
	Matrix temp;
	SetOrthoLH(temp, zRange, width, height, zNear, zFar);
	return temp;
}

Math::Matrix Math::MatrixOrthoRH(ZRange::TYPE zRange, float width, float height, float zNear, float zFar)
{
	Matrix temp;
	SetOrthoRH(temp, zRange, width, height, zNear, zFar);
	return temp;
}
